package fr.uca.iut.clfreville2.teaiswarm.fragment

import android.os.Bundle
import android.view.View
import android.widget.Button
import android.widget.TextView
import androidx.core.os.bundleOf
import androidx.fragment.app.Fragment
import androidx.lifecycle.lifecycleScope
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.RecyclerView
import fr.uca.iut.clfreville2.teaiswarm.*
import fr.uca.iut.clfreville2.teaiswarm.model.FileType
import fr.uca.iut.clfreville2.teaiswarm.model.RepositoryIdentifier
import fr.uca.iut.clfreville2.teaiswarm.model.VersionedFile
import kotlinx.coroutines.launch
import kotlin.io.path.Path

const val FILE_PATH = "file_path"

class RepositoryDetailsFragment : Fragment(R.layout.repository_details) {

    private val service = TeaIsWarm.service
    private lateinit var repositoryName: TextView
    private lateinit var repositoryDescription: TextView
    private lateinit var activity: Button
    private lateinit var versionedFiles: RecyclerView

    private lateinit var currentRepositoryOwner: String
    private lateinit var currentRepositoryName: String
    private var currentFilePath: String? = null

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        repositoryName = view.findViewById(R.id.repository_detail_name)
        repositoryDescription = view.findViewById(R.id.repository_detail_description)
        activity = view.findViewById(R.id.repository_detail_activity)

        val bundle = requireArguments()
        currentRepositoryOwner = bundle.getString(REPOSITORY_OWNER)!!
        currentRepositoryName = bundle.getString(REPOSITORY_NAME)!!
        currentFilePath = bundle.getString(FILE_PATH)

        repositoryName.text = currentRepositoryName

        versionedFiles = view.findViewById(R.id.versioned_files_view)
        lifecycleScope.launch {
            val repos = service.listFileContents(RepositoryIdentifier(currentRepositoryOwner, currentRepositoryName), currentFilePath ?: "")
            versionedFiles.adapter = FileListAdapter(repos) {
                    file -> adapterOnClick(file)
            }
        }

        activity.setOnClickListener {
            findNavController().navigate(R.id.activity_list_fragment, bundle)
        }
    }

    private fun adapterOnClick(file: VersionedFile) {
        val bundle = bundleOf(
            REPOSITORY_OWNER to currentRepositoryOwner,
            REPOSITORY_NAME to currentRepositoryName,
            FILE_PATH to Path(currentFilePath ?: "", file.name).toString()
        )
        when (file.type) {
            FileType.FILE -> {
                findNavController().navigate(R.id.code_view_fragment, bundle)
            }
            FileType.DIR -> {
                findNavController().navigate(R.id.repository_details_fragment, bundle)
            }
            else -> {}
        }
    }
}